#! /usr/bin/env python

import unittest, os, re
import shutil, tempfile
import time

import base, util

cvs_importer=['../cvs-importer']
svn_importer=cvs_importer+['--scm=svn']


class CVS_Repository:
	def __init__(self):
		self.tmpdir = tempfile.mktemp()
		self.reposdir = os.path.join(self.tmpdir, 'repos')
		os.mkdir(self.tmpdir, 0700)
		os.mkdir(self.reposdir, 0700)
		util.safesystem('cvs','-d',self.reposdir,'init')
		self.importer = cvs_importer + ['-d', self.reposdir]

	def __del__(self):
		if hasattr(self,'tmpdir') and os.path.exists(self.tmpdir):
			print 'CVS_Repository: removing',self.tmpdir
			shutil.rmtree(self.tmpdir)
	
	def checkout(self,name,ver=None):
		self.codir = os.path.join(self.tmpdir, 'co')
		if os.path.exists(self.codir):
			shutil.rmtree(self.codir)
		os.mkdir(self.codir, 0700)
		if ver:
			tagver = name + '-' + re.sub('[^-_0-9a-zA-Z]', '_', ver)
			util.safesystem('cvs','-d',self.reposdir,'co','-P','-d',self.codir,'-r',tagver,name)
		else:
			util.safesystem('cvs','-d',self.reposdir,'co','-P','-d',self.codir,name)
	
	def get_ignore_list(self, path, fn='.cvsignore'):
		filename = os.path.join(self.codir, path, fn)
		if not os.path.exists(filename):
			return []
		return open(filename,"r").read().split()
	
	def get_log(self, path):
		olddir = os.getcwd()
		os.chdir(self.codir)
		try:
			return util.getoutput('cvs','log',path)
		finally:
			os.chdir(olddir)

	def is_binary(self, path):
		olddir = os.getcwd()
		os.chdir(self.codir)
		try:
			return util.getoutput('cvs','status',path).strip().find("-kb")>=0
		finally:
			os.chdir(olddir)


class CVSTestCase(base.CommonTests, base.TestCase):
	def setUp(self):
		base.CommonTests.setUp(self)
		self.repos = CVS_Repository()
		self.addflag=0
	def tearDown(self):
		base.CommonTests.tearDown(self)
		del self.repos
	def do_addver(self, *args, **kw):
		if self.addflag:
			time.sleep(1.01) #cvs doesn't notice the files are different if size and timestamp are the same
		self.addflag=1
		base.CommonTests.do_addver(self, *args, **kw)

	def test_generated_pat_global_cvsignore(self):
		self.do_addver("1.0", add=["foo.py","foo.pyc","dir1/bar.py","dir1/bar.pyc"])
		self.do_import('--use-global-cvsignore')
		self.reset_tarballer()
		self.do_addver("1.0", add=["foo.py","dir1/bar.py"])
		self.do_verify_all()
		self.vfailUnlessEqual(self.repos.get_ignore_list(os.curdir), [])
		self.vfailUnlessEqual(self.repos.get_ignore_list("dir1"), [])
		self.repos.checkout('CVSROOT')
		self.vfailUnlessEqual(self.repos.get_ignore_list(os.curdir, "cvsignore"), ["*.py[co]"])
	
	def test_global_cvsignore_already_exists(self):
		self.repos.checkout('CVSROOT')
		filename = os.path.join(self.repos.codir, 'cvsignore')
		f = open(filename, 'w')
		f.write("testpat\n")
		f.close()
		olddir = os.getcwd()
		os.chdir(self.repos.codir)
		try:
			util.safesystem('cvs','add','cvsignore')
			util.safesystem('cvs','ci','-m','test')
		finally:
			os.chdir(olddir)
		self.do_addver("1.0", add=["foo.py"])
		self.do_import('--use-global-cvsignore')
		self.repos.checkout('CVSROOT')
		self.vfailUnlessListEqual(self.repos.get_ignore_list(os.curdir, "cvsignore"), ["testpat", "*.py[co]"])


if __name__ == '__main__':
	unittest.main()
